# tika
from tika import parser

# dataclasses
from dataclasses import dataclass
from tempfile import TemporaryDirectory

# images and numpy
import numpy as np
from PIL import Image

# mmm
import os
import time
import json
import shutil
import requests
import subprocess
from glob import glob
from typing import List, Dict
from tempfile import TemporaryDirectory

# constants to identify the pdffigures executable
# __file__ = "./simon/toolkits/documents.py"
FILEDIR = os.path.dirname(os.path.abspath(__file__))
# path to java
# TODO change this at will or put in .env 
JARDIR = os.path.abspath( 
    os.path.join(FILEDIR, "pdffigures2.jar"))
JAVADIR = os.path.realpath(shutil.which("java"))

def parse_figures(target):
    """Extract figures from PDF file with pdffigures2.

    Parameters
    ----------
    target : str
        The file to get figures from.

    Returns
    -------
    list
        A list dictionaries containing figures, their captions, and a numpy array for the figure.
    """

    # get full path of target
    target_path = os.path.abspath(target)

    # store temporary directory
    wd = os.getcwd()
    # create and change to temporary directory
    with TemporaryDirectory() as tmpdir:
        # change into temproary directory and extract figures
        os.chdir(tmpdir)
        subprocess.check_output(f"java -jar {JARDIR} -g meta -m fig {target_path} -q", shell=True)

        # read the metadata file
        meta_path = glob("meta*.json")[0]
        with open(meta_path, 'r') as df:
            meta = json.load(df)

        # open each of the images as numpy
        for figure in meta["figures"]:
            img = Image.open(figure["renderURL"])
            figure["render"] = np.array(img)
            img.close()

    # change directory back
    os.chdir(wd)

    return meta["figures"]

